///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Text;
using ILNumerics.Drawing.Lighting;
using System.Collections; 

namespace ILNumerics.Drawing.Collections {
    /// <summary>
    /// collection of 8 lights, individual configurable
    /// </summary>
    public class ILLightCollection 
        : IEnumerable {

        #region eventing
        public EventHandler Changed; 
        protected void OnChanged() {
            if (Changed != null) {
                Changed(this, new EventArgs()); 
            }
        }
        #endregion

        #region attributes
        List<ILLight> m_lights; 
        #endregion

        #region properties
        /// <summary>
        /// set: enable/disable all lights; get: return true, if at least one light is enabled, false else 
        /// </summary>
        public bool Enabled {
            get { 
                foreach (ILLight l in m_lights)
                    if (l.Enabled) return true; 
                return false; 
            }
            set { 
                foreach (ILLight l in m_lights) 
                    l.Enabled = value; 
            }
        }
        #endregion

        #region constructors
        public ILLightCollection () {
            m_lights = new List<ILLight>(8);
            for (int i = 0; i < 8; i++) {
                ILLight light = new ILLight(i); 
                m_lights.Add(light); 
                light.Changed += new EventHandler(lightChanged); 
            }
        }
        #endregion

        #region public interface
        /// <summary>
        /// get light at specific index
        /// </summary>
        /// <param name="i">index of light, valid: 0...7</param>
        /// <returns>Light at index i</returns>
        public ILLight this[int i] {
            get {
                return m_lights[i]; 
            }
        }
        #endregion

        #region private helpers
        protected void lightChanged (object sender, EventArgs args) {
            OnChanged(); 
        }
        #endregion



        #region IEnumerable Member

        public IEnumerator GetEnumerator() {
            for (int i = 0; i < m_lights.Count; i++) {
                yield return m_lights[i]; 
            }
        }

        #endregion
    }
}
