///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using ILNumerics.Drawing.Graphs;
using System.Collections.Generic; 
using ILNumerics.Drawing.Shapes; 

namespace ILNumerics.Drawing.Interfaces {
    /// <summary>
    /// interface for a painter factory, capable of creating device dependend axes and graphs 
    /// </summary>
    /// <remarks>The interface is implemented by (device dependent) panel implementations. 
    /// It is used to create the ILPanel's object hirarchy on startup. </remarks>
    internal interface IILCreationFactory { 
        /// <summary>
        /// create an axis 
        /// </summary>
        /// <param name="name">axis name</param>
        /// <param name="clippingView">clipping of the host</param>
        /// <param name="parameters">additional parameters</param>
        /// <returns>newly created axis object</returns>
        ILAxis CreateAxis(AxisNames name, ILClippingData clippingView, params object[] parameters);
        /// <summary>
        /// Create a new specific ILGraph object (device dependent)
        /// </summary>
        /// <param name="data">numeric data for the new graph</param>
        /// <param name="graphType">type of graph</param>
        /// <param name="additionalParams">additional parameter</param>
        /// <returns>newly created graph instance</returns>
        ILGraph CreateGraph(ILInArray<float> data, GraphType graphType, params ILArray<float>[] additionalParams);
        /// <summary>
        /// create a new device dependant scene graph
        /// </summary>
        /// <returns>scene graph</returns>
        ILSceneGraph CreateSceneGraph();

        /// <summary>
        /// create vertex renderer for rendering arbitrary vertices. 
        /// </summary>
        /// <param name="vertexType">underlying vertex type (value type)</param>
        /// <returns>vertex renderer</returns>
        ILVertexRenderer CreateVertexRenderer(Type vertexType, ILShape shape);
    }
}
