///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Drawing;

namespace ILNumerics.Drawing.Labeling {
    /// <summary>
    /// general label, mainly used for ILAxis
    /// </summary>
    public class ILLabel : ILLabelingElement {

        #region attributes
        private int m_padding; 
        private LabelAlign m_align; 
        internal Point m_position = new Point();
        private bool m_visible; 
        #endregion

        #region properties 
        /// <summary>
        /// get/set visibility for the label
        /// </summary>
        public bool Visible {
            get { return m_visible; }
            set { m_visible = value; }
        }
        /// <summary>
        /// get/ set the position for the label
        /// </summary>
        public Point Position {
            get {
                return m_position; 
            }
            set {
                m_position = value; 
            }
        }
        /// <summary>
        /// Alignment along the axis value range. Possible values: lower, center, upper
        /// </summary>
        public LabelAlign Alignment {
            get {
                return m_align; 
            }
            set {
                m_align = value; 
                OnChanged(); 
            }
        }
        /// <summary>
        /// Get/ set the padding used to seperate the label from the elements around it
        /// </summary>
        public int Padding {
            get {
                return m_padding; 
            }
            set {
                m_padding = value; 
                OnChanged(); 
            }
        }
        #endregion

        #region constructor
        /// <summary>
        /// create a label for rendering
        /// </summary>
        public ILLabel (ILPanel panel) 
            : base(panel,new Font(FontFamily.GenericSansSerif,10.0f),Color.DarkBlue) {
            m_align = LabelAlign.Center; 
            m_padding = 3;
            m_visible = true; 
        }
        #endregion

        #region abstract overrides

        /// <summary>
        /// draws the whole rendering queue
        /// </summary>
        public override void Draw(ILRenderProperties p) {
            if (!m_visible || String.IsNullOrEmpty(m_expression)) return; 
            if (m_expression != m_cachedExpression) 
                interprete(m_expression); 
            m_renderer.Begin(p);
            Point renderPoint = offsetAlignment(m_size, m_position);
            m_renderer.Draw(m_renderQueue, renderPoint ,m_orientation, m_color); 
            m_renderer.End(p); 
        }

        #endregion

        #region private helper


        #endregion
    }
}
