///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Text;

namespace ILNumerics.Drawing.Labeling {
    /// <summary>
    /// attribute used to identify IILTextRenderer types within assemblies 
    /// </summary>
    public class ILRendererAttribute : Attribute {
        /// <summary>
        /// the type of graphic device, the renderer is able to handle
        /// </summary>
        public readonly GraphicDeviceType GraphicDeviceType; 
        /// <summary>
        /// short name describing the renderer
        /// </summary>
        public readonly string Name; 
        /// <summary>
        /// long name describing the renderer
        /// </summary>
        public readonly string NameLong; 
        /// <summary>
        /// determine, if this renderer should be used as default 
        /// renderer for the graphic device and the coord system
        /// </summary>
        public readonly bool IsDefault; 

        /// <summary>
        /// Detemine the coord system the renderer is based on
        /// </summary>
        public readonly CoordSystem Coords; 

        /// <summary>
        /// Constructor for a new attribute describing non-default renderer
        /// </summary>
        /// <param name="type">graphic device type </param>
        /// <param name="Name">short name</param>
        /// <param name="NameLong">long name</param>
        public ILRendererAttribute(GraphicDeviceType type, string Name, string NameLong) {
            this.GraphicDeviceType = type; 
            this.Name = Name; 
            this.NameLong = NameLong; 
            this.IsDefault = false;
            this.Coords = CoordSystem.Screen; 
        }
        /// <summary>
        /// Constructor creating a new renderer attribute
        /// </summary>
        /// <param name="type">graphic device type </param>
        /// <param name="Name">short name</param>
        /// <param name="NameLong">long name</param>
        /// <param name="Default">'default' flag</param>
        public ILRendererAttribute(GraphicDeviceType type, string Name, string NameLong, bool Default, CoordSystem coords) {
            this.GraphicDeviceType = type; 
            this.Name = Name; 
            this.NameLong = NameLong; 
            this.IsDefault = Default;
            this.Coords = coords;
        }
    }
}
