///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing; 

namespace ILNumerics.Drawing.Lighting {
    public class ILMaterial : ICloneable {

        #region attributes
        byte m_shininess;
        Color m_specular;
        Color m_emission; 
        #endregion

        #region properties
        /// <summary>
        /// shape/intensity for specular reflection, range: 1...128
        /// </summary>
        public byte Shininess {
            get { return m_shininess; }
            set { m_shininess = value; }
        }
        /// <summary>
        /// color for specular reflection
        /// </summary>
        public Color Specular {
            get { return m_specular; }
            set { m_specular = value; }
        }
        /// <summary>
        /// color for emissive reflection
        /// </summary>
        public Color Emission {
            get { return m_emission; }
            set { m_emission = value; }
        }
        #endregion

        #region public interface
        /// <summary>
        /// construct new material object, initialize default values
        /// </summary>
        public ILMaterial() {
            m_shininess = (byte)96;
            m_specular = Color.FromArgb(255, Color.Gray);
            m_emission = Color.FromArgb(255, Color.Black); 
        }

        public object Clone() {
            return MemberwiseClone(); 
        }
        #endregion

        #region private helpers

        #endregion
    }
}
