///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing; 
using ILNumerics.Drawing.Labeling; 

namespace ILNumerics.Drawing.Marker {
    public abstract class ILBitmapMarkerShape : ILMarkerShape {

        #region attributes
        protected Bitmap m_bitmap; 
        #endregion

        #region constructors
        internal ILBitmapMarkerShape (ILPanel panel, Bitmap bitmap) : base(panel) {
            m_bitmap = bitmap;     
        }
        #endregion

        #region properties
        public Bitmap Bitmap {
            get {
                return m_bitmap; 
            }
            set {
                m_bitmap = value; 
                storeBitmap(m_bitmap); 
            }
        }
        #endregion

        #region public interface
        public string Hash() {
            // create hash based on the width, height & middle scanline of bitmap
            StringBuilder sb = new StringBuilder(
                        String.Format("M:{0}W{1}H{2}:","BMP",m_bitmap.Width,m_bitmap.Height)); 
            int h = m_bitmap.Height / 2; 
            for (int x = 0; x < Math.Min(m_bitmap.Width,30);x++) {
                sb.Append(m_bitmap.GetPixel(x,h).ToArgb().ToString());
            }
            return sb.ToString(); 
        }

        #endregion

        #region private helper
        protected virtual void storeBitmap(Bitmap bitmap) {
            string key = Hash(); 
            ILTextureManager mana = m_panel.TextureManager; 
            mana.StoreTextureItem(key,m_bitmap); 
        }
        #endregion
    }
}
