///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

#pragma warning disable 1591

using System;
using System.Collections.Generic;
using System.Text;
using ILNumerics; 

namespace ILNumerics.Drawing.Misc {
    
    public class ILActionRamp : List<ILActionRampElement> {

        public static ILActionRamp s_OverrideRamp;
        public static ILActionRamp Override {
            get {
                if (s_OverrideRamp == null) {
                    using (ILScope.Enter()) {
                        s_OverrideRamp = new ILActionRamp();
                        ILArray<float> tmp = ILMath.sin(ILMath.linspace<float>(-ILMath.pi / 1.7, ILMath.pi / 1.7, 20));
                        tmp -= tmp[0];
                        tmp /= tmp["end"];
                        foreach (float val in tmp) {
                            s_OverrideRamp.Add(new ILActionRampElement(val, .02f));
                        }
                    }
                }
                return s_OverrideRamp;
            }
        }
        public static ILActionRamp s_softRamp;
        public static ILActionRamp Soft {
            get {
                if (s_softRamp == null) {
                    using (ILScope.Enter()) {
                        s_softRamp = new ILActionRamp();
                        ILArray<float> tmp = ILMath.sin(ILMath.linspace<float>(-ILMath.pif / 2f, ILMath.pif / 2f, 10f));
                        tmp += 1;
                        tmp /= 2;
                        foreach (float val in tmp) {
                            s_softRamp.Add(new ILActionRampElement(val, .02f));
                        }
                    }
                }
                return s_softRamp;
            }
        }
        public static ILActionRamp s_hardRamp;
        public static ILActionRamp Hard {
            get {
                if (s_hardRamp == null) {
                    using (ILScope.Enter()) {
                        s_hardRamp = new ILActionRamp();
                        ILArray<float> tmp = ILMath.sin(ILMath.linspace<float>(0.0, 1.0, 10.0));
                        foreach (float val in tmp) {
                            s_hardRamp.Add(new ILActionRampElement(val, .02f));
                        }
                    }
                }
                return s_hardRamp;
            }
        }
        public static ILActionRamp s_linearRamp;
        public static ILActionRamp Linear {
            get {
                if (s_linearRamp == null) {
                    using (ILScope.Enter()) {
                        s_linearRamp = new ILActionRamp();
                        for (int i = 0; i < 10; i++) {
                            s_linearRamp.Add(new ILActionRampElement(i / 9.0f, 0.02f));
                        }
                    }
                }
                return s_linearRamp;
            }
        }
        public static ILActionRamp s_noRamp;
        public static ILActionRamp NoRamp {
            get {
                if (s_noRamp == null) {
                    s_noRamp = new ILActionRamp();
                    s_noRamp.Add(new ILActionRampElement(1, 0));
                }
                return s_noRamp;
            }
        }

        public ILActionRamp() {}
    }
}
