///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Text;
using ILNumerics.Drawing.Graphs; 

namespace ILNumerics.Drawing.Misc {
    public class ILGraphComparer : IComparer<ILGraph> {

        ILCamera m_camera;

        public ILCamera Camera {
            get { return m_camera; }
            set { m_camera = value; }
        } 

        public ILGraphComparer () {}



        #region IComparer<ILGraph> Member

        /// <summary>
        /// sort graphs regarding their Z order 
        /// </summary>
        /// <param name="x">first graph</param>
        /// <param name="y">second graph</param>
        /// <returns></returns>
        public int Compare(ILGraph x, ILGraph y) {
            if (m_camera.LooksFromTop)
                return Comparer<float>.Default.Compare(x.Limits.CenterF.Z,y.Limits.CenterF.Z); 
            else     
                return Comparer<float>.Default.Compare(y.Limits.CenterF.Z,x.Limits.CenterF.Z); 
            // if finer control over sorting is needed, try the following code instead! 
            //ILPoint3Df cent1 = x.Limits.CenterF; 
            //ILPoint3Df cent2 = y.Limits.CenterF; 
            //float distX = cent1.X - m_camera.X; 
            //float distY = cent1.Y - m_camera.Y; 
            //float distZ = cent1.Z - m_camera.Z; 
            //float dist1 = distX * distX + distY * distY + distZ * distZ; 
            //distX = cent2.X - m_camera.X;
            //distY = cent2.Y - m_camera.Y;
            //distZ = cent2.Z - m_camera.Z;
            //float dist2 = distX * distX + distY * distY + distZ * distZ; 
            //if (dist2 > dist1) {
            //    return 1; 
            //} else if (dist2 == dist1) {
            //    return 0; 
            //} else 
            //    return -1; 
        }
        #endregion
    }
}
