///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Text;

namespace ILNumerics.Drawing {
    /// <summary>
    /// extended drawing properties
    /// </summary>
    /// <remarks>the class collects flags for special
    /// rendering states (vector exports, picking).
    /// </remarks>
    public class ILRenderProperties {
        /// <summary>
        /// current camera 
        /// </summary>
        public ILCamera Camera;
        /// <summary>
        /// current clipping (view limits) data
        /// </summary>
        public bool Clipping;
        /// <summary>
        /// if not null, the rendering should be done offscreen via this graphics object
        /// </summary>
        public System.Drawing.Graphics Graphics;
        /// <summary>
        /// if set to true while rendering, the render process will be cancelled 
        /// </summary>
        public bool Canceled = false;
        /// <summary>
        /// inform content object of the reason for rendering, see the <see cref="ILNumerics.Drawing.RenderReason">enumeration</see> for possible values.
        /// </summary>
        internal RenderReason Reason;
        /// <summary>
        /// measures the absolute used up region while rendering, initiallized to int.MaxValue
        /// </summary>
        public int MinX;
        /// <summary>
        /// measures the absolute used up region while rendering, initiallized to int.MinValue
        /// </summary>
        public int MaxX;
        /// <summary>
        /// measures the absolute used up region while rendering, initiallized to int.MaxValue
        /// </summary>
        public int MinY;
        /// <summary>
        /// measures the absolute used up region while rendering, initiallized to int.MinValue
        /// </summary>
        public int MaxY;
        /// <summary>
        /// Some rendering settings require multiple rendering passes. Those are counted here.
        /// </summary>
        public int PassCount = 0;

        /// <summary>
        /// Resets this instance.
        /// </summary>
        /// <returns>Instance reset to default values.</returns>
        public ILRenderProperties Reset() {
            PassCount = 0;
            Reason = RenderReason.PaintEvent;
            MinX = int.MaxValue;
            MinY = int.MaxValue;
            MaxX = int.MinValue;
            MaxY = int.MinValue;
            return this; 
        }
    }
}
