///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using OpenTK;
using OpenTK.Platform;
using OpenTK.Graphics; 
using OpenTK.Graphics.OpenGL; 
using OpenTK.Graphics.OpenGL.Enums; 
using System.Drawing;
using ILNumerics.Drawing.Interfaces; 
using ILNumerics.Drawing.Labeling; 
using ILNumerics.Drawing; 

namespace ILNumerics.Drawing.Platform.OpenGL {
    /// <summary>
    /// OpenGL implementation for ILAxis 
    /// </summary>
    public abstract class ILOGLAxis : ILAxis, IDisposable {

        #region attributes
        private int m_primitivCount;
        #endregion

        #region constructor
        internal ILOGLAxis (AxisNames name, ILClippingData clippingView,
                            ILLayoutData layoutData,
                            ILOGLPanel panel)
            : base (name,clippingView,layoutData,panel) {
            m_invalidated = true;
            panel.GraphicsDeviceCreated += new ILGraphicsDeviceCreatedEvent(initialize);
            panel.GraphicsDeviceReset += new ILGraphicsDeviceResetEvent(configure);
        }
        #endregion

        #region helper functions
        private void initialize(object sender, EventArgs eventArgs) {
            if (m_panel == null || m_panel.GetDeviceContext() == null)
                return; 
            Invalidate(); 
            // todo: setup font + sprite objects for OpenGL ?
            m_primitivCount = countPrimitves(); 
            //m_labeledTicks.TextRenderer = m_panel.TextRendererManager.CreateInstance(
            //            "ILNumerics.Drawing.TextRenderer.ILGDIRenderer" ,null); 
            //m_label.TextRenderer = m_panel.TextRendererManager.CreateInstance(
            //            "ILNumerics.Drawing.TextRenderer.ILGDIRenderer" ,null); 
        }

        void configure(object sender, EventArgs eventArgs) {
        }

#if DEBUG
        private void debub_write_mat(string prefix) {
            double[] modmat = new double[16]; 
            GL.GetDouble(GetPName.ProjectionMatrix,modmat); 
            System.Diagnostics.Debug.Write(prefix + new ILArray<double>(modmat,4,4).ToString()); 
        }
#endif

        protected void ConfigureOGLLineProperties(LineStyle style, int width) { 
            int stipFact = 0; 
            short stipple = ILPanel.StippleFromLineStyle(style,ref stipFact); 
            GL.LineWidth(width); 
            if (style == LineStyle.Solid || style == LineStyle.None) {
                GL.Disable(EnableCap.LineStipple); 
            } else {
                GL.Enable(EnableCap.LineStipple); 
                GL.LineStipple (stipFact,stipple); 
            }
        }
        #endregion 

        #region ILAxis abstract overrides
        public override void Dispose()
        {
            m_invalidated = true;
            base.Dispose();
        }

        public override void PrepareMeshes(ILRenderProperties p) {
            return; 
        }

        public override void PrepareLabels(ILRenderProperties p) {
            return; 
        }

        #endregion
         

    }
}
